@@ -1,6 +1,6 @@
 package DBIx::Class::DeploymentHandler::Dad;
 {
-  $DBIx::Class::DeploymentHandler::Dad::VERSION = '0.002000';
+  $DBIx::Class::DeploymentHandler::Dad::VERSION = '0.002100';
 }
 
 # ABSTRACT: Parent class for DeploymentHandlers
@@ -45,15 +45,19 @@ sub _build_schema_version { $_[0]->schema->schema_version }
 
 sub install {
   my $self = shift;
-  log_info { 'installing version ' . $self->to_version };
+
+  my $version = (shift @_ || {})->{version} || $self->to_version;
+  log_info { "installing version $version" };
   croak 'Install not possible as versions table already exists in database'
     if $self->version_storage_is_installed;
 
-  my $ddl = $self->deploy({version=>$self->to_version});
+  $self->txn_do(sub {
+     my $ddl = $self->deploy({ version=> $version });
 
-  $self->add_database_version({
-    version     => $self->to_version,
-    ddl         => $ddl,
+     $self->add_database_version({
+       version     => $self->to_version,
+       ddl         => $ddl,
+     });
   });
 }
 
@@ -61,18 +65,20 @@ sub upgrade {
   log_info { 'upgrading' };
   my $self = shift;
   my $ran_once = 0;
-  while ( my $version_list = $self->next_version_set ) {
-    $ran_once = 1;
-    my ($ddl, $upgrade_sql) = @{
-      $self->upgrade_single_step({ version_set => $version_list })
-    ||[]};
+  $self->txn_do(sub {
+     while ( my $version_list = $self->next_version_set ) {
+       $ran_once = 1;
+       my ($ddl, $upgrade_sql) = @{
+         $self->upgrade_single_step({ version_set => $version_list })
+       ||[]};
 
-    $self->add_database_version({
-      version     => $version_list->[-1],
-      ddl         => $ddl,
-      upgrade_sql => $upgrade_sql,
-    });
-  }
+       $self->add_database_version({
+         version     => $version_list->[-1],
+         ddl         => $ddl,
+         upgrade_sql => $upgrade_sql,
+       });
+     }
+  });
 
   log_warn { 'no need to run upgrade' } unless $ran_once;
 }
@@ -81,13 +87,15 @@ sub downgrade {
   log_info { 'downgrading' };
   my $self = shift;
   my $ran_once = 0;
-  while ( my $version_list = $self->previous_version_set ) {
-    $ran_once = 1;
-    $self->downgrade_single_step({ version_set => $version_list });
+  $self->txn_do(sub {
+     while ( my $version_list = $self->previous_version_set ) {
+       $ran_once = 1;
+       $self->downgrade_single_step({ version_set => $version_list });
 
-    # do we just delete a row here?  I think so but not sure
-    $self->delete_database_version({ version => $version_list->[0] });
-  }
+       # do we just delete a row here?  I think so but not sure
+       $self->delete_database_version({ version => $version_list->[0] });
+     }
+  });
   log_warn { 'no version to run downgrade' } unless $ran_once;
 }
 
@@ -145,6 +153,10 @@ See L<DBIx::Class::DeploymentHandler::HandlesDeploy/upgrade_single_step>.
 
 See L<DBIx::Class::DeploymentHandler::HandlesDeploy/downgrade_single_step>.
 
+=head2 txn_do
+
+See L<DBIx::Class::DeploymentHandler::HandlesDeploy/txn_do>.
+
 =head1 ORTHODOX METHODS
 
 These methods are not actually B<required> as things will probably still work
@@ -216,8 +228,12 @@ The version (defaults to schema's version) to migrate the database to
 
  $dh->install
 
-Deploys the current schema into the database.  Populates C<version_storage> with
-C<version> and C<ddl>.
+or
+
+ $dh->install({ version => 1 })
+
+Deploys the requested version into the database  Version defaults to
+L</schema_version>.  Populates C<version_storage> with C<version> and C<ddl>.
 
 B<Note>: you typically need to call C<< $dh->prepare_deploy >> before you call
 this method.
