@@ -1,6 +1,6 @@
 package DBIx::Class::DeploymentHandler::HandlesDeploy;
 {
-  $DBIx::Class::DeploymentHandler::HandlesDeploy::VERSION = '0.002000';
+  $DBIx::Class::DeploymentHandler::HandlesDeploy::VERSION = '0.002100';
 }
 use Moose::Role;
 
@@ -20,6 +20,8 @@ requires 'upgrade_single_step';
 requires 'prepare_downgrade';
 requires 'downgrade_single_step';
 
+requires 'txn_do';
+
 1;
 
 # vim: ts=2 sw=2 expandtab
@@ -133,6 +135,12 @@ Optionally return C<< [ $ddl, $upgrade_sql ] >> where C<$ddl> is the DDL for
 that version of the schema and C<$upgrade_sql> is the SQL that was run to
 upgrade the database.
 
+=head2 txn_do
+
+ $dh->txn_do(sub { ... })
+
+Wrap the passed coderef in a transaction (if transactions are enabled.)
+
 =head1 AUTHOR
 
 Arthur Axel "fREW" Schmidt <frioux+cpan@gmail.com>
